@@ -839,10 +839,11 @@ class PGLoader(threading.Thread):
                 self.prepare_processing()
                 self.process()
                 self.finish_processing()
-                
+
             except Exception, e:
                 # resources get freed in self.terminate()
                 self.log.error(e)
+                raise
 
             self.terminate()
             return
@@ -982,8 +983,13 @@ class PGLoader(threading.Thread):
         thread will read the input file and distribute the processing
         on a round-robin fashion"""
         self.prepare_processing()
+
+        try:
+            from RRRtools import RRReader
+        except ImportError, e:
+            raise PGLoader_Error, \
+                  "Please upgrade to python 2.4 or newer: %s" % e
         
-        from tools import RRReader
         queues   = {}
         locks    = {}
         sem      = threading.BoundedSemaphore(self.section_threads)
@@ -1045,7 +1051,7 @@ class PGLoader(threading.Thread):
         c = 0               # current
         p = c               # previous
         
-        for line, columns in self.reader.readlines():
+        for offsets, line, columns in self.reader.readlines():
             if p != c:
                 self.log.debug("read %d lines, queue to thread %s" % (n, c))
 
@@ -1064,7 +1070,7 @@ class PGLoader(threading.Thread):
                     self.log.debug("locks[%d].acquire" % c)
                     locks[c].acquire()
                 
-            queues[c].append((line, columns))
+            queues[c].append((offsets, line, columns))
             n += 1
             p = c
             c = (n / self.rrqueue_size) % self.section_threads
@@ -1129,8 +1135,8 @@ class PGLoader(threading.Thread):
         """ return next line from either self.queue or self.reader """
 
         if self.queue is None:
-            for line, columns in self.reader.readlines():
-                yield line, columns
+            for offsets, line, columns in self.reader.readlines():
+                yield offsets, line, columns
 
             return
 
@@ -1139,8 +1145,8 @@ class PGLoader(threading.Thread):
 
             if len(self.queue) > 0:
                 self.log.debug("processing queue")
-                for line, columns in self.queue.readlines():
-                    yield line, columns
+                for offsets, line, columns in self.queue.readlines():
+                    yield offsets, line, columns
 
             self.lock.release()
 
@@ -1211,7 +1217,9 @@ class PGLoader(threading.Thread):
         if self.udcs:
             dudcs = dict(self.udcs)
 
-        for line, columns in self.readlines():
+        for offsets, line, columns in self.readlines():
+            self.log.debug('offsets %s', offsets)
+
             if self.blob_cols is not None:
                 columns, rowids = self.read_blob(line, columns)
 
@@ -1279,12 +1287,12 @@ class PGLoader(threading.Thread):
                     
             if not DRY_RUN:
                 self.db.copy_from(self.table, self.columnlist,
-                                  data, line, self.reject)
+                                  data, line, offsets, self.reject)
 
         if not DRY_RUN:
             # we may need a last COPY for the rest of data
             self.db.copy_from(self.table, self.columnlist,
-                              None, None, self.reject, EOF = True)
+                              None, None, None, self.reject, EOF = True)
 
         return
 
